"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Box = void 0;
const mithril_1 = __importDefault(require("mithril"));
const classnames_1 = __importDefault(require("classnames"));
const Component_1 = require("../common/Component");
require("./box.style.css");
const Toolbar_1 = require("../toolbar/Toolbar");
const Button_1 = require("../controls/button/Button");
const Datagrid_1 = require("../datagrid/Datagrid");
const Tree_1 = require("../tree/Tree");
const Container_1 = require("../common/Container");
const Text_1 = require("../controls/text/Text");
const Space_1 = require("../controls/space/Space");
/**
 * The Box class represents a flexible container that uses the flexbox model in CSS.
 * It allows for the creation of a dynamic layout where each cell can have its own functionality,
 * including a title, collapsible/expandable behavior, and the ability to hide/show content.
 * Cells can contain plain text, html or other components like Box, Toolbar, Table etc.
 */
class Box extends Container_1.Container {
    constructor(attrs, parent) {
        if (attrs && Array.isArray(attrs.content)) {
            const { content } = attrs, rest = __rest(attrs, ["content"]);
            super(rest);
            this._boxes = [];
            content.forEach(attributes => {
                this.add(attributes);
            });
        }
        else {
            super(attrs);
            this._boxes = [];
        }
        this._parent = parent;
        this._createHeader();
        this._computeFlex();
    }
    /**
     * Returns one of the nested cells by its id
     * @param id id of the cell to get
     * @returns Box container associated with passed id, or undefined if there is no nested box with requested id
     */
    getContainer(id) {
        let container = this._boxes.find(x => x.getId() === id);
        if (container) {
            return container;
        }
        this._boxes.find(x => {
            container = x.getContainer(id);
            return container;
        });
        return container;
    }
    /**
     * Returns array of all nested boxes on every level down until there is another box attached as content
     */
    getAllContainers() {
        const containers = [];
        containers.push(...this._boxes);
        this._boxes.forEach(container => {
            containers.push(...container.getAllContainers() || []);
        });
        return containers;
    }
    /**
     * Add new cell to the box
     * @param attributes configuration of the new box, or component instance to add inside new box
     * @returns Newly created box
     */
    add(attributes) {
        if (this.getContent() !== undefined) {
            throw new Error('Cannot add container because box contains component attached!');
        }
        else {
            let boxAttributes;
            let box;
            if (attributes instanceof Box) {
                this._boxes.push(attributes);
                box = attributes;
            }
            else {
                if (attributes instanceof Component_1.Component) {
                    boxAttributes = {
                        content: attributes
                    };
                }
                else {
                    boxAttributes = attributes;
                }
                box = new Box(boxAttributes, this);
                this._boxes.push(box);
            }
            this._computeFlex();
            mithril_1.default.redraw();
            return box;
        }
    }
    /**
     * Get information about children boxes direction
     * @returns direction that box cells are organized into
     */
    getDirection() {
        var _a;
        return (_a = this.attrs.direction) !== null && _a !== void 0 ? _a : 'column';
    }
    /**
     * Change direction of the children boxes
     * @param direction direction to organize children into
     */
    setDirection(direction) {
        this.attrs.direction = direction;
        this._computeFlex();
        mithril_1.default.redraw();
    }
    /**
     * Makes box visible/invisible on the page as whole (header and content)
     * @param value true to hide the box
     */
    setHidden(value) {
        this.attrs.hidden = value;
        mithril_1.default.redraw();
    }
    /**
     * Get text displays as the box header title
     * @returns text of the box header
     */
    getHeader() {
        return this.attrs.header;
    }
    /**
     * Changes text displays as box header title
     * @param header text to display as header title
     */
    setHeader(header) {
        this.attrs.header = header;
        this._computeFlex();
        mithril_1.default.redraw();
    }
    /**
     * Get style of gaps between each container inside the box
     * @returns name of gap style currently used
     */
    getGap() {
        var _a;
        return this.attrs.gap || ((_a = this._parent) === null || _a === void 0 ? void 0 : _a.getGap()) || 'small';
    }
    /**
     * Changes style of gap spaces between each container inside the box
     * @param gap name of gap style to use
     */
    setGap(gap) {
        this.attrs.gap = gap;
        mithril_1.default.redraw();
    }
    /**
     * Hide box content. Remains only header line to display
     */
    collapse() {
        this.attrs.collapsed = true;
        this.header.getItem('toggle').setIcon(this._getCollapsingIcon());
        this._computeFlex();
        mithril_1.default.redraw();
    }
    /**
     * Show box content.
     */
    expand() {
        this.attrs.collapsed = false;
        this.header.getItem('toggle').setIcon(this._getCollapsingIcon());
        this._computeFlex();
        mithril_1.default.redraw();
    }
    /**
     * Hide/show box content.
     */
    toggle() {
        if (this.isCollapsed()) {
            this.expand();
        }
        else {
            this.collapse();
        }
    }
    /**
     * Get information that box content is visible or not
     * @returns true when box content is visibled
     */
    isCollapsed() {
        return Boolean(this.attrs.collapsed);
    }
    view() {
        var _a;
        if (this.attrs.hidden) {
            return mithril_1.default.fragment({ key: this.getId() }, []);
        }
        return (0, mithril_1.default)('div', {
            key: this.getId(),
            className: (0, classnames_1.default)([
                'webcraft_box',
                `webcraft_box--${((_a = this._parent) === null || _a === void 0 ? void 0 : _a.getGap()) || this.getGap()}`,
                this.isCollapsed() ? 'webcraft_box--collapsed' : null,
                this._boxes.length === 0 ? 'webcraft_box--container' : null,
                this._getOverflowCss(),
                this.attrs.classnames,
                this._getBoxCss(),
                this.content instanceof Box || this._boxes.length ? null : 'webcraft_box--ui',
            ]),
            style: Object.assign(Object.assign({}, this._getSizes()), { flex: this._flex }),
            oncreate: (vnode) => {
                this.events.emit('onCreate', vnode.dom);
            },
            onupdate: (vnode) => {
                this.events.emit('onUpdate', vnode.dom);
            },
        }, [
            this.header.getItems().length ? this.header.view() : null,
            !this.isCollapsed() && this._getView({
                className: (0, classnames_1.default)([
                    'webcraft_box_content',
                    `webcraft_flex--${this.getDirection()}`,
                    this._getContentCss(),
                ]),
                style: {
                    flexWrap: this.attrs.flexWrap === false ? 'nowrap' : undefined,
                }
            }),
        ]);
    }
    _getBoxCss() {
        const css = [];
        if (this.header.getItems().length && this._boxes.length) {
            css.push('webcraft_box--space');
        }
        return css;
    }
    _getContentCss() {
        const css = [];
        if (this.header.getItems().length && this._boxes.length) {
            css.push('webcraft_box--border-x');
            css.push('webcraft_box--border-bottom');
        }
        return css;
    }
    _getOverflowCss() {
        const content = this.getContent();
        if (content instanceof Datagrid_1.Datagrid || content instanceof Tree_1.Tree) {
            return 'webcraft_overflow--hidden';
        }
    }
    _computeFlex() {
        const getFlex = () => {
            if (this.isCollapsed()) {
                return '0 0 content';
            }
            if (this.attrs.flex) {
                return this.attrs.flex;
            }
            const contentStyle = this._getSizes();
            if (this.content instanceof Component_1.Component && this.content.fitContainer) {
                return '0 0 content';
            }
            if (contentStyle.width) {
                return '0 0 content';
            }
            if (contentStyle.maxWidth) {
                return '1 1 content';
            }
            if (this._boxes.map(item => item.content).every(Boolean)) {
                return '1 0 content';
            }
            return '1 1 0%';
        };
        this._flex = getFlex();
    }
    _getChildren() {
        if (this._boxes.length > 0) {
            return this._boxes.map(item => item.view());
        }
        return super._getChildren();
    }
    _createHeader() {
        const toolbarItems = [];
        if (this.attrs.header) {
            toolbarItems.push(new Text_1.Text({
                value: this.attrs.header,
                classNames: 'webcraft_header_title',
            }));
        }
        if (this.attrs.collapsable) {
            toolbarItems.push(new Space_1.Space());
            toolbarItems.push(new Button_1.Button({
                id: 'toggle',
                round: true,
                size: 'small',
                color: 'secondary',
                style: 'plain',
                icon: this._getCollapsingIcon(),
                events: {
                    click: () => {
                        this.toggle();
                    }
                }
            }));
        }
        this.header = new Toolbar_1.Toolbar({
            classNames: 'webcraft_box_header',
            items: toolbarItems,
        });
    }
    _getSizes() {
        if (this.isCollapsed()) {
            return {};
        }
        const style = Object.assign({}, (this.attrs.sizes || {}));
        ['width', 'height', 'minWidth', 'maxWidth', 'minHeight', 'maxHeight', 'padding', 'margin'].forEach(type => {
            if (typeof style[type] === 'number') {
                style[type] += 'px';
            }
        });
        return style;
    }
    _getCollapsingIcon() {
        var _a;
        return {
            'row': {
                'true': 'icon-right-open',
                'false': 'icon-left-open',
            },
            'column': {
                'true': 'icon-down-open',
                'false': 'icon-up-open',
            },
        }[((_a = this._parent) === null || _a === void 0 ? void 0 : _a.getDirection()) || 'column'][String(this.isCollapsed())];
    }
}
exports.Box = Box;
